@@ -6,7 +6,6 @@ from math import inf, sqrt, prod
 from numbers import Number
 from fractions import Fraction
 
-
 class Matrix:
     """
     Matrix class.
@@ -51,6 +50,20 @@ class Matrix:
             out[i] += self.__class__.print_post
         return '\n'.join(out)
 
+    def _repr_mimebundle_(self, **kwargs):
+        return {
+            "text/plain": repr(self),
+            "text/latex": "$\\displaystyle" + self.latex() + "$"
+        }
+
+    def latex(self) -> str:
+        "Produce LaTeX code for the matrix."
+        res = "\\begin{pmatrix}\n"
+        for row in self.matrix:
+            res += " & ".join(map(str, row)) + '\\\\\n'
+        res += '\\end{pmatrix}'
+        return res
+
     def __len__(self):
         return self.cols * self.rows
 
@@ -92,7 +105,7 @@ class Matrix:
             if isinstance(j, int):
                 cols = [j]
             elif isinstance(j, list):
-                cols = i
+                cols = j
             else:
                 cols = range(self.cols)[j]
             for i, ii in zip(cols, range(len(cols))):
@@ -106,6 +119,73 @@ class Matrix:
                 i, j = divmod(k, self.cols)
                 self.matrix[i][j] = value[k]
 
+    def __delitem__(self, item):
+        if isinstance(item, tuple):
+            i, j = item
+            if isinstance(i, int):
+                rows = [i]
+            elif isinstance(i, list):
+                rows = set(i)
+            else:
+                rows = range(self.rows)[i]
+            if isinstance(j, int):
+                cols = [j]
+            elif isinstance(j, list):
+                cols = set(j)
+            else:
+                cols = range(self.cols)[j]
+            if len(cols) == self.cols:
+                self.delete_rows(rows)
+                return
+            elif len(rows) == self.rows:
+                self.delete_columns(cols)
+                return
+        raise ValueError("Can only delete entire rows or entire columns!")
+            
+    def delete_rows(self, rows: int|list) -> None:
+        "Deletes a row or a list of rows."
+        if isinstance(rows, int):
+            rows = [ rows ]
+        rows = list(map(lambda x: x % self.rows, rows))
+        rows = set(rows)
+        if len(rows) >= self.rows:
+            raise ValueError("Cannot delete all rows.")
+        for i in reversed(sorted(rows)):
+            del self.matrix[i]
+        self.rows -= len(rows)
+    
+    def delete_columns(self, cols: int|list) -> None:
+        "Deletes a column or a list of columns."
+        if isinstance(cols, int):
+            cols = [ cols ]
+        cols = list(map(lambda x: x % self.cols, cols))
+        cols = set(cols)
+        if len(cols) >= self.cols:
+            raise ValueError("Cannot delete all columns.")
+        for i in range(self.rows):
+            for j in reversed(sorted(cols)):
+                del self.matrix[i][j]
+        self.cols -= len(cols)
+    
+    def append_row(self, row: list|tuple, ring = None) -> None:
+        "Append a row."
+        if not isinstance(row, list|tuple) or len(row) != self.cols:
+            raise ValueError("Length does not match the number of columns.")
+        if ring:
+            row =list(map(ring, row))
+        self.matrix.append(row)
+        self.rows += 1
+
+    def append_column(self, col: list|tuple, ring = None) -> None:
+        "Append a column."
+        if not isinstance(col, list|tuple) or len(col) != self.rows:
+            raise ValueError("Length does not match the number of rows.")
+        if ring:
+            col =list(map(ring, col))
+        for i in range(self.rows):
+            self.matrix[i].append(col[i])
+        self.cols += 1
+
     def __eq__(self, other):
         if not isinstance(other, self.__class__):
             return False
@@ -127,6 +207,11 @@ class Matrix:
         tmp.map(func)
         return tmp
 
+    def _guess_zero(self):
+        zero = 0 * self.matrix[0][0]
+        one = zero**0
+        return zero, one
+
     def norm2(self) -> float:
         "Squared Frobenius/Euclidean norm."
         return sum(sum(x*x for x in row) for row in self.matrix)
@@ -203,19 +288,20 @@ class Matrix:
 
     def rref(self) -> "Matrix":
         "Compute the reduced echelon form of a matrix M."
+        one = self._guess_zero()[1]
         n, m = self.cols, self.rows
         R = self[:, :]
         i = 0
         for j in range(n):
-            if not R[i, j]: # search for a nonzero entry in the present column
-                for ii in range(i+1,m):
+            if not R[i, j]:  # search for a nonzero entry in the present column
+                for ii in range(i+1, m):
                     if R[ii, j]:
                         R[i, :], R[ii, :] = R[ii, :], R[i, :]  # swap rows
                         break
                 else:
                     continue  # all entries are zero
-            if R[i, j] != 1:
-                R[i, :] = 1 / R[i, j] * R[i, :]  # make the pivot one
+            if R[i, j] != one:
+                R[i, :] = R[i, j]**-1 * R[i, :]  # make the pivot one
             for ii in range(m):  # remove the column entries above/below the pivot
                 if i == ii:
                     continue
@@ -228,24 +314,25 @@ class Matrix:
 
     def det(self) -> int:
         "Compute the determinant of a matrix M."
+        zero, one = self._guess_zero()
         if self.rows != self.cols:
             raise ValueError("Matrix must be square!")
         n, m = self.cols, self.rows
         R = self[:, :]
-        D = 1
+        D = one
         i = 0
         for j in range(n):
             if not R[i, j]:  # search for a nonzero entry in the present column
                 for ii in range(i+1, m):
                     if R[ii, j]:
-                        D *= -1
+                        D *= -one
                         R[i, :], R[ii, :] = R[ii, :], R[i, :]  # swap rows
                         break
                 else:
-                    return 0  # all entries are zero
-            if R[i, j] != 1:
+                    return zero  # all entries are zero
+            if R[i, j] != one:
                 D *= R[i, j]
-                R[i, :] = 1 / R[i, j] * R[i, :]  # make the pivot one
+                R[i, :] = R[i, j]**-1 * R[i, :]  # make the pivot one
             for ii in range(i+1, n):  # remove the column entries below the pivot
                 if i == ii:
                     continue
@@ -254,20 +341,51 @@ class Matrix:
             i += 1
         return D
 
+    def rank(self) -> int:
+        "Compute the rank of a matrix M."
+        MM = self.rref()
+        for i in range(self.rows-1,-1,-1):
+            if MM[i,:]:
+                return i+1
+        return 0
+
     def inv(self) -> "Matrix":
         "Compute the inverse of a square matrix M."
         if self.rows != self.cols:
             raise ValueError("Matrix must be square!")
+        zero, one = self._guess_zero()
         n = self.cols
-        MM = self.__class__([[0 for _ in range(2*n)] for _ in range(n)])
+        MM = self.__class__([[zero for _ in range(2*n)] for _ in range(n)])
         for i in range(n):
-            MM[i, n+i] = 1
+            MM[i, n+i] = one
         MM[:, 0:n] = self
         MM = MM.rref()
         if not prod(MM[i, i] for i in range(n)):
             raise ValueError("Matrix is not invertible!")
         return MM[:, n:]
 
+    def solve(self, b: "Matrix", ring = None) -> "Matrix":
+        "Solve the linear system with given inhomogenous vector."
+        if isinstance(b, list|tuple):
+            b = Matrix(b, ring = ring)
+        if self.rows != b.rows or b.cols != 1:
+            raise ValueError("Matrix dimensions do not match.")
+        A = self.zeros(self.rows, self.cols + 1) # extended coefficient matrix
+        A[:, 0:self.cols] = self
+        A[:, self.cols] = b
+        A = A.rref()
+        solution = self.zeros(self.cols, 1)
+        for i in range(A.rows-1, -1, -1):
+            if not any(A.matrix[i][:-1]):
+                if A.matrix[i][-1]:
+                    return None  # Not solvable
+            else:
+                for j in range(A.cols - 1):
+                    if A.matrix[i][j]:
+                        break # leading nonzero coefficient
+                solution[j]= A.matrix[i][-1]
+        return solution
+
     def is_unimodular(self) -> bool:
         "Test if the matrix is unimodular."
         if self.rows != self.cols:
@@ -285,10 +403,9 @@ class Matrix:
             n, m = self.cols, self.rows
         elif not n:
             n = m
-        try:
-            zero = 0 * self[0]
-        except:
-            zero = 0
+        if n < 1 or m < 1:
+            raise ValueError(f"Matrix dimensions {m}x{n} must be positive!")
+        zero = self._guess_zero()[0]
         return self.__class__([[zero for j in range(n)] for i in range(m)])
 
     def eye(self, m: int = None, n: int = None):
@@ -301,30 +418,29 @@ class Matrix:
             n, m = self.cols, self.rows
         elif not n:
             n = m
-        try:
-            zero = 0 * self[0]
-        except:
-            zero = 0
-        try:
-            one = zero**0
-        except:
-            one = 1
+        if n < 1 or m < 1:
+            raise ValueError(f"Matrix dimensions {m}x{n} must be positive!")
+        zero, one = self._guess_zero()
         return self.__class__([[delta(i, j) for j in range(n)] for i in range(m)])
 
 
-def zeros(m: int, n: int = None, zero=0) -> "Matrix":
+def zeros(m: int, n: int = None, zero=0, ring=None) -> "Matrix":
     "Returns a zero matrix of the given dimension."
     if not n:
         n = m
-    return Matrix([[zero for j in range(n)] for i in range(m)])
+    if n < 1 or m < 1:
+        raise ValueError(f"Matrix dimensions {m}x{n} must be positive!")
+    return Matrix([[zero for j in range(n)] for i in range(m)], ring=ring)
 
 
-def eye(m: int, n: int = None) -> "Matrix":
+def eye(m: int, n: int = None, zero=0, one=1, ring=None) -> "Matrix":
     "Returns an identity matrix of the given dimension."
     def delta(i, j):
         if i == j:
-            return 1
-        return 0
+            return one
+        return zero
     if not n:
         n = m
-    return Matrix([[delta(i, j) for j in range(n)] for i in range(m)])
+    if n < 1 or m < 1:
+        raise ValueError(f"Matrix dimensions {m}x{n} must be positive!")
+    return Matrix([[delta(i, j) for j in range(n)] for i in range(m)], ring=ring)
